export declare type RunFunction = () => void | number | Promise<void | number>;
export declare type BootstrapFunction = RunFunction;
export declare type ErrorHandlerFunction = (e: Error) => void | Promise<void>;
/**
 * Options for bootstrapping
 */
export interface BootstrapOptions {
    register?: RunFunction;
    run: RunFunction;
    onComplete?: RunFunction;
    onFinally?: RunFunction;
    teardown?: RunFunction;
    shouldExitOnError?: boolean;
    errorHandler?: ErrorHandlerFunction;
}
/**
 * Default options
 */
export declare const defaultOptions: BootstrapOptions;
export declare class Bootstrap {
    /**
     * Should the process exit when the register/run throw an exception
     */
    shouldExitOnError: boolean;
    /**
     * Register function runs before entrypoint, and is useful
     *	for setting up services etc
     */
    protected register: RunFunction;
    /**
     * Teardown function runs before the application exits
     */
    protected teardown: RunFunction;
    /**
     * onComplete runs when the run function returns (.then())
     */
    protected onComplete: RunFunction;
    /**
     * onError runs when the run function throws an exception (.catch())
     */
    protected onError: ErrorHandlerFunction;
    /**
     * onFinally runs when the run function stops (.finally())
     */
    protected onFinally: RunFunction;
    /**
     * @internal
     */
    protected isBooted: boolean;
    constructor(options?: BootstrapOptions);
    /**
     * Boot and run a function
     *
     * @param run Run function
     */
    boot(run: RunFunction): void;
    /**
     * Boot (async)
     */
    bootAsync(): Promise<void>;
    /**
     * Gracefully exit
     *
     * @param code Exit code
     */
    exit(code?: number): void;
    setRegister(register: RunFunction): void;
    setTeardown(teardown: RunFunction): void;
    setOnComplete(onComplete: RunFunction): void;
    setOnError(onError: ErrorHandlerFunction): void;
    setOnFinally(onFinally: RunFunction): void;
}
/**
 * Bootstrap a function
 *
 * @param options Bootstrapping options
 */
export declare function bootstrap(options: BootstrapOptions): Bootstrap;
/**
 * Returns a promise that will resolve after bootstrapping is complete.
 * 	You will then be responsible for calling your main entrypoint
 * 	function manually (usually after awaiting this function)
 *
 * @param register Register function
 */
export declare function bootstrapPromise(register: RunFunction): Promise<void>;
