"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrapPromise = exports.bootstrap = exports.Bootstrap = exports.defaultOptions = void 0;
/**
 * Default options
 */
exports.defaultOptions = {
    register: null,
    run: null,
    onComplete: () => { },
    onFinally: () => { },
    shouldExitOnError: true,
    errorHandler: console.error
};
class Bootstrap {
    constructor(options = exports.defaultOptions) {
        /**
         * Should the process exit when the register/run throw an exception
         */
        this.shouldExitOnError = true;
        /**
         * Register function runs before entrypoint, and is useful
         *	for setting up services etc
         */
        this.register = () => __awaiter(this, void 0, void 0, function* () { });
        /**
         * Teardown function runs before the application exits
         */
        this.teardown = () => __awaiter(this, void 0, void 0, function* () { });
        /**
         * onComplete runs when the run function returns (.then())
         */
        this.onComplete = () => __awaiter(this, void 0, void 0, function* () { });
        /**
         * onError runs when the run function throws an exception (.catch())
         */
        this.onError = console.error;
        /**
         * onFinally runs when the run function stops (.finally())
         */
        this.onFinally = () => __awaiter(this, void 0, void 0, function* () { });
        /**
         * @internal
         */
        this.isBooted = false;
        process.on('SIGINT', this.exit.bind(this));
        process.on('SIGTERM', this.exit.bind(this));
        process.on('exit', this.exit.bind(this));
        if (options.register) {
            this.setRegister(options.register);
        }
        if (options.onComplete) {
            this.setOnComplete(options.onComplete);
        }
        if (options.onFinally) {
            this.setOnFinally(options.onFinally);
        }
        if (options.teardown) {
            this.setTeardown(options.teardown);
        }
        if (options.shouldExitOnError !== null) {
            this.shouldExitOnError = options.shouldExitOnError;
        }
        if (options.errorHandler) {
            this.setOnError(options.errorHandler);
        }
    }
    /**
     * Boot and run a function
     *
     * @param run Run function
     */
    boot(run) {
        this.isBooted = true;
        Promise.resolve(this.register ? this.register() : null)
            .then(() => __awaiter(this, void 0, void 0, function* () { return yield Promise.resolve(run()); }))
            .then(() => __awaiter(this, void 0, void 0, function* () { return yield Promise.resolve(this.onComplete()); }))
            .catch((e) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            // Handle or log the error
            yield Promise.resolve(this.onError(e));
            if (this.shouldExitOnError) {
                this.exit.bind(this)((_a = e.code) !== null && _a !== void 0 ? _a : 1);
            }
        }))
            .finally(() => __awaiter(this, void 0, void 0, function* () { return yield Promise.resolve(this.onFinally()); }));
    }
    /**
     * Boot (async)
     */
    bootAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            this.shouldExitOnError = false;
            return yield new Promise((accept, reject) => {
                this.setOnError(reject);
                this.boot(() => __awaiter(this, void 0, void 0, function* () { return accept(); }));
            });
        });
    }
    /**
     * Gracefully exit
     *
     * @param code Exit code
     */
    exit(code = 1) {
        if (!this.isBooted) {
            process.exit(code);
        }
        this.isBooted = false;
        Promise.resolve(this.teardown()).finally(() => process.exit(code));
    }
    setRegister(register) {
        if (this.isBooted) {
            throw new Error('Cannot setRegister() after boot');
        }
        this.register = register.bind(this);
    }
    setTeardown(teardown) {
        if (this.isBooted) {
            throw new Error('Cannot setTeardown() after boot');
        }
        this.teardown = teardown.bind(this);
    }
    setOnComplete(onComplete) {
        if (this.isBooted) {
            throw new Error('Cannot setOnComplete() after boot');
        }
        this.onComplete = onComplete.bind(this);
    }
    setOnError(onError) {
        if (this.isBooted) {
            throw new Error('Cannot setOnError() after boot');
        }
        this.onError = onError.bind(this);
    }
    setOnFinally(onFinally) {
        if (this.isBooted) {
            throw new Error('Cannot setOnFinally() after boot');
        }
        this.onFinally = onFinally.bind(this);
    }
}
exports.Bootstrap = Bootstrap;
/**
 * Bootstrap a function
 *
 * @param options Bootstrapping options
 */
function bootstrap(options) {
    const b = new Bootstrap(Object.assign(Object.assign({}, exports.defaultOptions), options));
    b.boot(options.run);
    return b;
}
exports.bootstrap = bootstrap;
/**
 * Returns a promise that will resolve after bootstrapping is complete.
 * 	You will then be responsible for calling your main entrypoint
 * 	function manually (usually after awaiting this function)
 *
 * @param register Register function
 */
function bootstrapPromise(register) {
    return __awaiter(this, void 0, void 0, function* () {
        const b = new Bootstrap({
            register: register,
            run: () => __awaiter(this, void 0, void 0, function* () { }),
        });
        return yield b.bootAsync();
    });
}
exports.bootstrapPromise = bootstrapPromise;
//# sourceMappingURL=index.js.map